/*
 * VRC4172 PWM controller support for MC-R700
 */

/*
 * file: files.hpcmips
 *
 * device vrc4172pwm
 * attach vrc4172pwm at vrip
 * file arch/hpcmips/vr/vrc4172pwm.c		vrc4172pwm
 *
 *
 * file: kernel config file(like GENERIC)
 *
 * vrc4172pwm* at vrip? addr 0x15003880 size 0x08 # MCR-700 backlight control
 *
 */

#include <sys/param.h>
#include <sys/systm.h>
#include <sys/device.h>

#include <machine/config_hook.h>
#include <mips/cpuregs.h>
#include <machine/bus.h>
#include <machine/platid.h>
#define PLATID_DEFINE_MASK_NICKNAME
#include <machine/platid_mask.h>

#include <hpcmips/hpcmips/machdep.h>	/* hpcmips_verbose */
#include <hpcmips/vr/vripreg.h>
#include <hpcmips/vr/vripvar.h>
#include <hpcmips/vr/vrgiureg.h>

#include "locators.h"

#define LCDDUTYEN (0)

struct vrc4172pwm_softc {
  struct device sc_dev;
  bus_space_tag_t sc_iot;
  bus_space_handle_t sc_ioh;
};

static int  vrc4172pwm_match  __P((struct device *, struct cfdata *, void *));
static void vrc4172pwm_attach __P((struct device *, struct device *, void *));
static int  vrc4172pwm_hook   __P((void *, int, long, void *));

struct cfattach vrc4172pwm_ca = {
  sizeof(struct vrc4172pwm_softc), vrc4172pwm_match, vrc4172pwm_attach
};

static int
vrc4172pwm_match(struct device *parent, struct cfdata *cf, void *aux)
{
  if(platid_match(&platid, (platid_mask_t *)NEC_MCR_700)){
    return 2; /* 1st attach group of vrip */
  }

  return 0;
}

static void
vrc4172pwm_attach(struct device *parent, struct device *self, void *aux)
{
  struct vrip_attach_args *va = aux;
  struct vrc4172pwm_softc *sc = (void*)self;

  sc->sc_iot = va->va_iot;
  bus_space_map(sc->sc_iot, va->va_addr, va->va_size,
		0 /* no cache */, &sc->sc_ioh);

  printf("\n");

  config_hook(CONFIG_HOOK_POWERCONTROL,
	      CONFIG_HOOK_POWERCONTROL_LCDLIGHT,
	      CONFIG_HOOK_SHARE,
	      vrc4172pwm_hook,
	      sc);
}

static int
vrc4172pwm_hook(void *ctx, int type, long id, void *msg)
{
  struct vrc4172pwm_softc *sc = ctx;
  unsigned short val16;

  if(msg){
    /* resume */

    val16 = bus_space_read_2(sc->sc_iot, sc->sc_ioh, LCDDUTYEN);
    bus_space_write_2(sc->sc_iot, sc->sc_ioh, LCDDUTYEN, val16 | 0x0001);

  }else{
    /* suspend */

    val16 = bus_space_read_2(sc->sc_iot, sc->sc_ioh, LCDDUTYEN);
    bus_space_write_2(sc->sc_iot, sc->sc_ioh, LCDDUTYEN, val16 & ~0x0001);
  }

  return (0);
}
